/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyClockItem, SGraphyScene } from "@sybotan-web/graphy/lib";
import Axios from "axios";
// @ts-ignore
import pako from "pako";
import { JsonData } from "./types/JsonData";
import { Space } from "./types/Space";
import { Column } from "./types/Column";
import { Wall } from "./types/Wall";
import { VirtualWall } from "./types/VirtualWall";
import { Door } from "./types/Door";
import { Casement } from "./types/Casement";

/**
 * 楼层场景
 *
 * @author  郝建龙
 */
export class FloorScene extends SGraphyScene {
    /** item数据  */
    data: JsonData | null = null;
    /** 是否显示空间  */
    isShowSpace = true;
    /** 是否展示墙体  */
    isShowWallList = true;
    /** 是否展示设备  */
    isShowEquipmentList = true;
    /** 是否展示虚拟墙  */
    isShowVrtualWallList = true;

    /**
     *  @param  data    绘制空间地图得所有参数
     */
    constructor() {
        super();
        let click = new SGraphyClockItem(null, 300, 300);
        this.addItem(click);
    } // Constructor()

    /**
     *  获取底图文件
     *
     *  @param  url     请求数据文件路径
     */
    urlGetData(url: string) {
        let that = this;
        return new Promise((relove, reject) => {
            Axios({
                method: "get",
                url: url,
                data: {},
                responseType: "blob"
            })
                .then((res: any) => {
                    let blob = res.data;
                    this.zipToJson(blob)
                        .then((jsonData: any) => {
                            that.addAllItems(jsonData);
                            relove();
                        })
                        .catch((error: any) => {
                            console.log(error);
                        });
                    // console.log(reader)
                })
                .catch((res: any) => {
                    console.log(res);
                });
        });
    } // Function urlGetData

    /**
     *  解压数据
     *
     *  @param  blob     文件
     */
    zipToJson(blob: any): any {
        let reader = new FileReader();
        reader.readAsBinaryString(blob);
        let that = this;
        return new Promise(resolve => {
            reader.onload = (readerEvt: any) => {
                let binaryString = readerEvt.target.result;
                //解压数据
                let base64Data = btoa(binaryString);
                let unGzipData = pako.unzip(base64Data);
                that.data = unGzipData.EntityList.Elements;
                resolve(unGzipData);
            };
        });
    } // Function zipToJson

    /**
     *  增添所有绘制item(地图);
     *
     *  @param  data    itemList对象
     */
    addAllItems(data: JsonData) {
        this.addCasementItems(data.Casement);
        this.addSpaceItems(data.Spaces);
        this.addColumnItems(data.Columns);
        this.addWallItems(data.Walls);
        this.addVirtualWallItems(data.VirtualWalls);
        this.addDoorItems(data.Doors);
    } // Function addAllItemList

    /**
     *  添加所有空间到scene 中
     *
     *  @param  space   空间list
     */
    addSpaceItems(space: Space): void {} // Function addSpaceList

    /**
     *  添加所有设备到scene 中
     *
     *  @param  column   柱子list
     */
    addColumnItems(column: Column): void {} // Function addColumnListList

    /**
     *  添加所有设备到scene 中
     *
     *  @param  wall   墙list
     */
    addWallItems(wall: Wall): void {} // Function addWallList

    /**
     *  添加所有虚拟墙到scene 中
     *
     *  @param  virtualWall   虚拟墙list
     */
    addVirtualWallItems(virtualWall: VirtualWall): void {} // Function addVirtualWallList

    /**
     *  添加所有位置标签到scene 中
     *
     *  @param  doors   门list
     */
    addDoorItems(door: Door): void {} // Function addDoorList

    /**
     *  添加所有位置标签到scene 中
     *
     *  @param  windows   窗户list
     */
    addCasementItems(casement: Casement): void {} // Function addWindowList
    /**
     *  扩大数组中相应字段一定倍数
     *
     *  @param  arr     itemList
     *  @param  k       倍数
     *  @param  name    字段key值
     */
    changeMap(arr: any, k: any, name: any) {
        let data = arr.map((items: any) => {
            if (items[name] && items[name].length) {
                items[name].map((children: any) => {
                    if (Array.isArray(children)) {
                        return children.map(res => {
                            res.Y = res.Y * k;
                            return res;
                        });
                    } else {
                        children.Y = children.Y * k;
                        return children;
                    }
                });
            }
        });
        return data;
    }
} // Class FloorScene
